@@ -1,4 +1,4 @@
-from typing import List, Set
+from typing import List, Set, Union
 from astroid import nodes
 from astroid.inference import infer_name
 from pylint.checkers import BaseChecker
@@ -8,6 +8,7 @@ from pylint.interfaces import IAstroidChecker
 iterable_types = (nodes.Tuple, nodes.List, nodes.Set, )
 
 def get_children_recursive(node: nodes.NodeNG):
+    """Get children of a node."""
     for child in node.get_children():
         yield child
         yield from get_children_recursive(child)
@@ -29,9 +30,9 @@ class ForLoopChecker(BaseChecker):
             'Eager iteration of an iterable is inefficient.'
         ),
         'W8102': (
-            'Incorrect iterator method for dictionary, use .keys() or .values().',
+            'Incorrect iterator method for dictionary, use %s.',
             'incorrect-dictionary-iterator',
-            'Use keys() or values() instead of items() when not unpacking both.'
+            'Incorrect use of .items() when not unpacking key and value.'
         )
     }
 
@@ -69,8 +70,10 @@ class ForLoopChecker(BaseChecker):
                 return
             if not len(node.target.elts) == 2:
                 return
-            if any(el.name == '_' for el in node.target.elts if isinstance(el, nodes.AssignName)):
-                self.add_message("incorrect-dictionary-iterator", node=node.iter)
+            if isinstance(node.target.elts[0], nodes.AssignName) and node.target.elts[0].name == '_':
+                self.add_message("incorrect-dictionary-iterator", node=node.iter, args=('values()', ))
+            if isinstance(node.target.elts[1], nodes.AssignName) and node.target.elts[1].name == '_':
+                self.add_message("incorrect-dictionary-iterator", node=node.iter, args=('keys()', ))
 
         else:
             return
@@ -88,14 +91,14 @@ class LoopInvariantChecker(BaseChecker):
     priority = -1
     msgs = {
         'W8201': (
-            'Consider moving this statement outside of the for-loop.',
+            'Consider moving this expression outside of the loop.',
             'loop-invariant-statement',
-            'Eager iteration of an iterable is inefficient.'
+            'None of the variables referred to in this expression change within the loop.'
         ),
         'W8202': (
             'Lookups of global names within a loop is inefficient, copy to a local variable outside of the loop first.',
             'loop-invariant-global-usage',
-            'Eager iteration of an iterable is inefficient.'
+            'Global name lookups in Python are slower than local names.'
         ),
     }
 
@@ -104,6 +107,7 @@ class LoopInvariantChecker(BaseChecker):
         self._loop_level = 0
         self._loop_assignments: List[Set[str]] = []
         self._loop_names: List[List[nodes.Name]] = []
+        self._ignore: List[nodes.NodeNG] = []
 
     @checker_utils.check_messages("loop-invariant-statement")
     def visit_for(self, node: nodes.For) -> None:
@@ -114,9 +118,25 @@ class LoopInvariantChecker(BaseChecker):
         elif isinstance(node.target, nodes.AssignName):
             self._loop_assignments.append({node.target.name})
         self._loop_names.append([])
+        self._ignore.append(node.iter)
+
+    @checker_utils.check_messages("loop-invariant-statement")
+    def visit_while(self, node: nodes.While) -> None:
+        """Visit while loop bodies."""
+        self._loop_level += 1
+        self._loop_names.append([])
+        self._loop_assignments.append(set())
+        self._ignore.append(node.test)
 
     @checker_utils.check_messages("loop-invariant-statement")
     def leave_for(self, node: nodes.For) -> None:
+        self._leave_loop(node)
+
+    @checker_utils.check_messages("loop-invariant-statement")
+    def leave_while(self, node: nodes.While) -> None:
+        self._leave_loop(node)
+
+    def _leave_loop(self, node: Union[nodes.For, nodes.While]) -> None:
         """Drop loop level."""
         self._loop_level -= 1
         assigned_names = self._loop_assignments.pop()
@@ -131,13 +151,13 @@ class LoopInvariantChecker(BaseChecker):
                     for child in get_children_recursive(cur_node):
                         if isinstance(child, nodes.Name) and child.name in assigned_names:
                             is_variant = True
-                    if not is_variant:
+                    if not is_variant and not isinstance(cur_node, nodes.Keyword):
                         invariant_node = cur_node
                         cur_node = cur_node.parent
                     else:
                         break
 
-                if invariant_node:
+                if invariant_node and invariant_node not in self._ignore:
                     self.add_message("loop-invariant-statement", node=invariant_node)
 
 
@@ -151,6 +171,15 @@ class LoopInvariantChecker(BaseChecker):
         if isinstance(target, nodes.AssignName):
             self._loop_assignments[-1].add(target.name)
 
+    def visit_augassign(self, node: nodes.AugAssign) -> None:
+        """Track assignments in loops."""
+        # we don't handle multiple assignment nor slice assignment
+        if not self._loop_assignments:
+            return # Skip when empty
+
+        if isinstance(node.target, nodes.AssignName):
+            self._loop_assignments[-1].add(node.target.name)
+
     @checker_utils.check_messages("loop-invariant-global-usage")
     def visit_name(self, node: nodes.Name) -> None:
         """Look for global names"""
@@ -163,7 +192,7 @@ class LoopInvariantChecker(BaseChecker):
         scope, stmts = node.lookup(node.name)
         if not isinstance(scope, nodes.Module):
             return
-        if node.name in scope.globals:
+        if node.name in scope.globals and isinstance(scope.globals[node.name], nodes.AssignName):
             if self._loop_level > 0:
                 self.add_message("loop-invariant-global-usage", node=node)
 
